
c     The program "reset_netCDF_value" will change a value (or a range
c     of values) for a particular variable on a netCDF file, to a new
c     value. Run the program with no arguments to view the syntax.
c     NOTE! use "nc_reset_value.sh" in this directory!
c     NOTE! no attributes such as "missing_value" will be changed, 
c           use "nc_reset_attr" for this purpose


      program reset_netCDF_value

      implicit none

      include 'netcdf.inc'
      include 'nc_var_dims.inc'

c     Variables:
      integer varsize,filesize,nc_status,type,length
      integer narg,x,y,t,zt,n_rep,start(4),count(4)
      integer nc_id,var_id
      real    fraction
      character*1 reply
      character*2 vartype
      character*30 varname,old_val,string_val,new_val
      character*120 filename

c     Variables/values:
      integer*2 i2_val_old,i2_lower,i2_upper,i2_val_new,i2_array(nx,ny)
      integer   i4_val_old,i4_lower,i4_upper,i4_val_new,i4_array(nx,ny)
      real      r4_val_old,r4_lower,r4_upper,r4_val_new,r4_array(nx,ny)
      real*8    r8_val_old,r8_lower,r8_upper,r8_val_new,r8_array(nx,ny)

c     Functions:
      integer str_len,iargc


      narg = iargc()
      if (narg .ne. 6) go to 999

c     Read command line arguments:
      call getarg(1,filename)
      call getarg(2,varname)
      call getarg(3,vartype)
      call getarg(4,old_val)
      call getarg(5,string_val)
      call getarg(6,new_val)


c     Set interval etc.:

      read(string_val,*)fraction
      if (fraction .lt. 0. .or. fraction .ge. 1.) go to 998
      write(*,*)

      if (vartype .eq. 'i2' .or. vartype .eq. 'i4') then
         read(old_val,*)i4_val_old
         read(new_val,*)i4_val_new
         if (i4_val_old .ge. 0) then
            i4_lower = nint((1. - fraction)*float(i4_val_old))
            i4_upper = nint((1. + fraction)*float(i4_val_old))
         else
            i4_lower = nint((1. + fraction)*float(i4_val_old))
            i4_upper = nint((1. - fraction)*float(i4_val_old))
         end if
         if (vartype .eq. 'i2') then
            if (i4_val_old .lt. -32768 .or. i4_val_new .lt. -32768 .or.
     >          i4_val_old .gt.  32767 .or. i4_val_new .gt.  32767)
     >         go to 997
               i2_val_old = i4_val_old
               i2_val_new = i4_val_new
            if (i4_lower .lt. -32768) then
               i2_lower = -32768
            else
               i2_lower = i4_lower
            endif
            if (i4_upper .gt. 32767) then
               i2_upper = 32767
            else
               i2_upper = i4_upper
            endif
            write(*,*)'All numbers in the interval from (inclusive)'
            write(*,*)'     ',i2_lower
            write(*,*)' to (inclusive)'
            write(*,*)'     ',i2_upper
            write(*,*)' will be replaced by'
            write(*,*)'     ',i2_val_new
         else
            write(*,*)'All numbers in the interval from (inclusive)'
            write(*,*)'     ',i4_lower
            write(*,*)' to (inclusive)'
            write(*,*)'     ',i4_upper
            write(*,*)' will be replaced by'
            write(*,*)'     ',i4_val_new
         end if

      else if (vartype .eq. 'r4') then
         read(old_val,*)r4_val_old
         read(new_val,*)r4_val_new
         if (r4_val_old .ge. 0.) then
            r4_lower = (1. - fraction)*r4_val_old
            r4_upper = (1. + fraction)*r4_val_old
         else
            r4_lower = (1. + fraction)*r4_val_old
            r4_upper = (1. - fraction)*r4_val_old
         end if
         write(*,*)'All numbers in the interval from (inclusive)'
         write(*,*)'     ',r4_lower
         write(*,*)' to (inclusive)'
         write(*,*)'     ',r4_upper
         write(*,*)' will be replaced by'
         write(*,*)'     ',r4_val_new

      else if (vartype .eq. 'r8') then
         read(old_val,*)r8_val_old
         read(new_val,*)r8_val_new
         if (r8_val_old .ge. dble(0.)) then
            r8_lower = dble(1. - fraction)*r8_val_old
            r8_upper = dble(1. + fraction)*r8_val_old
         else
            r8_lower = dble(1. + fraction)*r8_val_old
            r8_upper = dble(1. - fraction)*r8_val_old
         end if
         write(*,*)'All numbers in the interval from (inclusive)'
         write(*,*)'     ',r8_lower
         write(*,*)' to (inclusive)'
         write(*,*)'     ',r8_upper
         write(*,*)' will be replaced by'
         write(*,*)'     ',r8_val_new

      else
         go to 999
      end if

         write(*,*)
      write(*,'(a,$)')'Do you wish to continue? (y/n) '
      read(5,*)reply
         write(*,*)
      if (reply .ne. 'y' .and. reply .ne. 'Y') go to 996


      filesize = str_len(filename)
      varsize  = str_len(varname)

c     Open netCDF file:
      nc_status = NF_OPEN(filename(1:filesize), NF_WRITE, nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,10)

c     Get variable ID, type:
      nc_status = NF_INQ_VARID(nc_id, varname(1:varsize), var_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,11)
      nc_status = NF_INQ_VARTYPE(nc_id, var_id, type)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,12)

c     Check type:
      if ( (vartype .eq. 'i2' .and. type .ne. NF_SHORT) .or.
     >     (vartype .eq. 'i4' .and. type .ne. NF_INT) .or.
     >     (vartype .eq. 'r4' .and. type .ne. NF_FLOAT) .or.
     >     (vartype .eq. 'r8' .and. type .ne. NF_DOUBLE) ) go to 995

      start(1) = 1
      start(2) = 1
      count(1) = nx
      count(2) = ny
      count(3) = 1
      count(4) = 1


      do t = 1,max(1,n2)

         if (n2 .gt. 0) write(*,'(a,i5,$)')'  ... processing step no',t
         start(4) = t
         n_rep = 0


         do zt = 1,max(1,n1)

            if (n2 .eq. 0) then
               write(*,'(a,i5,$)')'  ... processing step no',zt
               n_rep = 0
            end if
            start(3) = zt


            if (vartype .eq. 'i2') then

               nc_status = NF_GET_VARA_INT2(nc_id, var_id, start, count,
     >                                      i2_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,1)

               do y = 1,ny
                  do x = 1,nx
                     if (i2_array(x,y) .ge. i2_lower .and.
     >                   i2_array(x,y) .le. i2_upper) then
                        n_rep         = n_rep + 1
                        i2_array(x,y) = i2_val_new
                     end if
                  end do
               end do

               nc_status = NF_PUT_VARA_INT2(nc_id, var_id, start, count,
     >                                      i2_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,2)


            else if (vartype .eq. 'i4') then

               nc_status = NF_GET_VARA_INT(nc_id, var_id, start, count,
     >                                     i4_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,3)

               do y = 1,ny
                  do x = 1,nx
                     if (i4_array(x,y) .ge. i4_lower .and.
     >                   i4_array(x,y) .le. i4_upper) then
                        n_rep         = n_rep + 1
                        i4_array(x,y) = i4_val_new
                     end if
                  end do
               end do

               nc_status = NF_PUT_VARA_INT(nc_id, var_id, start, count,
     >                                     i4_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,4)


            else if (vartype .eq. 'r4') then

               nc_status = NF_GET_VARA_REAL(nc_id, var_id, start, count,
     >                                      r4_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,5)

               do y = 1,ny
                  do x = 1,nx
                     if (r4_array(x,y) .ge. r4_lower .and.
     >                   r4_array(x,y) .le. r4_upper) then
                        n_rep         = n_rep + 1
                        r4_array(x,y) = r4_val_new
                     end if
                  end do
               end do

               nc_status = NF_PUT_VARA_REAL(nc_id, var_id, start, count,
     >                                      r4_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,6)


            else

               nc_status = NF_GET_VARA_DOUBLE(nc_id, var_id,
     >                                        start, count, r8_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,7)

               do y = 1,ny
                  do x = 1,nx
                     if (r8_array(x,y) .ge. r8_lower .and.
     >                   r8_array(x,y) .le. r8_upper) then
                        n_rep         = n_rep + 1
                        r8_array(x,y) = r8_val_new
                     end if
                  end do
               end do

               nc_status = NF_PUT_VARA_DOUBLE(nc_id, var_id,
     >                                        start, count, r8_array)
               if (nc_status .ne. NF_NOERR) call handle_err(nc_status,8)


            end if

            if (n2 .eq. 0) write(*,*)', replaced #',n_rep


         end do

         if (n2 .gt. 0) write(*,*)', replaced #',n_rep

      end do


c     Close file:
      nc_status = NF_CLOSE(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,91)


      open(41, file= 'ok_flag', status='unknown')
      close(41)


      go to 1000

 995  write(*,*)
      write(*,*)'Error in the program "reset_netCDF_value".'
      write(*,*)' Inconsistency in specification of variable type, ',
     >          'and actual type on the file.'
      write(*,*)'  specified type: ',vartype
      write(*,'(a,$)')'  actual type:   '
      if (type .eq. NF_SHORT) then
         write(*,*)'i2'
      else if (type .eq. NF_INT) then
         write(*,*)'i4'
      else if (type .eq. NF_FLOAT) then
         write(*,*)'r4'
      else if (type .eq. NF_DOUBLE) then
         write(*,*)'r8'
      end if
      write(*,*)
      write(*,*)'Terminates.'
      write(*,*)
      go to 1000

 996  write(*,*)
      write(*,*)'The program "reset_netCDF_value" terminates upon ',
     >          'request.'
      write(*,*)
      go to 1000

 997  write(*,*)
      write(*,*)'Error in the program "reset_netCDF_value".'
      write(*,*)' One of the values below are not in integer*2 ',
     >          'range: [-32768,32767]'
      write(*,*)'  old value: ',i4_val_old
      write(*,*)'  new value: ',i4_val_new
      write(*,*)
      write(*,*)'Terminates.'
      write(*,*)
      go to 1000

 998  write(*,*)
      write(*,*)'Error in the program "reset_netCDF_value".'
      write(*,*)' The specified fraction (5th arg.): ',fraction
      write(*,*)'  must be in the range [0,1>'
      write(*,*)' See the syntax and explanation below.'
      write(*,*)

 999  write(*,*)
      write(*,*)'Error in the program "reset_netCDF_value".'
      write(*,*)' SYNTAX ERROR.'
      write(*,*)'  Syntax:'
      write(*,*)' nc_reset_value <file> <var name> <type> <old value> ',
     >          '<fraction> <new value>'
      write(*,*)'  where'
      write(*,*)'   <file>      is the name of a netCDF file'
      write(*,*)'   <var name>  is the name of a variable on <file>;'
      write(*,*)'                for a global attribute, use "global"'
      write(*,*)'   <type>  is the variable type of <var name>;'
      write(*,*)'                i2 - integer,        2 bytes'
      write(*,*)'                i4 - integer,        4 bytes'
      write(*,*)'                r4 - floating point, 4 bytes'
      write(*,*)'                r8 - floating point, 8 bytes'
      write(*,*)'   <old value> is the old value that will be replaced'
      write(*,*)'   <fraction>  gives the interval that will be ',
     >          'replaced by <new value>;'
      write(*,*)'                when <old value> > 0, <new value> ',
     >          'will replace'
      write(*,*)'                all values in the interval'
      write(*,*)'                 (1-<fraction>)*<old value> to'
      write(*,*)'                 (1+<fraction>)*<old value>'
      write(*,*)'                (inclusive)'
      write(*,*)'   <new value> is the new value for <var name>'
      write(*,*)
      write(*,*)'Terminates.'
      write(*,*)

 1000 end
