
c     The program "nc_variable_dimensions" performs the following actions:
c      *reads a netCDF file which has a name given as input either
c        during run time, or by directing input through an ASCII file
c      *reads the x,y,t dimensional sizes of a variable; and the z size
c        (if present, otherwise set to 0)
c      *writes the dimensional sizes on the file "nc_var_dims.inc"


      program nc_variable_dimensions

      implicit none

      include 'netcdf.inc'

c     Variables:
      integer nc_id,var_id,dim_ids(4)
      integer n_dims,dims(4),n,nc_status,size
      character*30 varname
      character*120 filename

c     For command line arguments:
      integer iargc,narg

c     Function:
      integer str_len

      narg = iargc()

      if (narg .ne. 2) then
         write(*,*)
         write(*,*)'Syntax error.'
         write(*,*)'  Correct syntax:'
         write(*,*)'    nc_dims <filename> <var name>'
         write(*,*)'     * where "nc_var_dims" will write an "include',
     >             ' file" with the'
         write(*,*)'     dimensional sizes for a variable on ',
     >             'the specified netCDF file'
         write(*,*)'     * the "include file" will be named ',
     >             'nc_var_dims.inc'
         write(*,*)'     <filename>  is the name of the netCDF input ',
     >             'file'
         write(*,*)'     <var name>  is the name of a netCDF ',
     >             'variable on <filename>'
         write(*,*)'Terminates.'
         write(*,*)
         stop
      end if


c     Open netCDF file:
      call getarg(1,filename)
      size = str_len(filename)
      nc_status = NF_OPEN(filename(1:size), NF_NOWRITE, nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,251)

c     Set variable id:
      call getarg(2,varname)
      size = str_len(varname)
      nc_status = NF_INQ_VARID(nc_id, varname(1:size), var_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,252)

c     Read the variable's dimensions and dimensional sizes:
      nc_status = NF_INQ_VARNDIMS(nc_id, var_id, n_dims)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,253)
      if (n_dims .gt. 4) go to 999
      nc_status = NF_INQ_VARDIMID(nc_id, var_id, dim_ids)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,254)

c     Initialize:
      do n = 1,4
         dims(n) = 0
      end do

c     Read dimensional sizes:
      do n = 1,n_dims
         nc_status = NF_INQ_DIMLEN(nc_id, dim_ids(n), dims(n))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,261)
      end do
      dims(2) = max(1,dims(2))

      open(31, file= 'nc_var_dims.inc', status= 'unknown',
     >         access= 'sequential', form= 'formatted')

 101  format(6x,a19)
      write(31,101)'integer nx,ny,n1,n2'

 102  format(6x,a15,i6,a6,i6,a6,i6,a6,i6,a1)
      write(31,102)'parameter (nx= ',dims(1),', ny= ',dims(2),
     >                      ', n1= ',dims(3),', n2= ',dims(4),')'

      close(31)


      nc_status = NF_CLOSE(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,99)


      open(41, file= 'ok_flag', status='unknown')
      close(41)

      go to 1000

 999  write(*,*)
      write(*,*)'Error in the program "nc_var_dims".'
      write(*,*)' ...not prepared for this high a dimension:'
      write(*,*)'  max. dimension:   ',4
      write(*,*)'  present dimension:',n_dims
      write(*,*)'Terminates.'
      write(*,*)

 1000 end
